#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <ESP8266HTTPClient.h> 
#include <FS.h>
#include <ArduinoJson.h>
#include "Adafruit_TCS34725.h"
#include <Servo.h>
#include <Stepper_28BYJ.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

#define SERVO1_PIN 0
#define SERVO2_PIN 2
#define BUTTON_PIN 1

const char *AP_ssid = "Settings";
//const char *AP_password = "12345680";

String ssid[3];
String password[3];
int workDelay = 1;
String jsonConfig = "{}";

const int sleepConst = 60000000; //Постоянная для установки времении сна в минутах

// количество шагов на один оборот
const int stepPerRevolution = 4096;
int stepCount = 512;
int stepAngle = 45;

ESP8266WebServer HTTP(80);     // Определяем объект и порт сервера для работы с HTTP
//FtpServer ftpSrv;

  HTTPClient http;
  
  Adafruit_TCS34725 tcs = Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_700MS, TCS34725_GAIN_1X);
  
  LiquidCrystal_I2C lcd(0x27,16,2);
  
  Servo servo1;
  Servo servo2;
  
  Stepper_28BYJ stepper1(stepPerRevolution, 15, 13, 12, 14);
  
  int workMode=1;
  
  void setup() {
    
    Serial.swap();
    lcd.init();                      // Инициализация дисплея  
    lcd.backlight();                 // Подключение подсветки
    lcd.clear();
    lcd.setCursor(0,0);              // Установка курсора в начало первой строкия
    lcd.print("Select mode:");
    int timeOut=0;                  
    delay(4000);
    pinMode (BUTTON_PIN, INPUT);
    workMode=digitalRead(BUTTON_PIN);       //Проверка выставленного режима работы
    Serial.swap();
    Serial.begin(115200);
    lcd.setCursor(0,1); 
  if (workMode==1){
    lcd.print("work");
  }
  else{
    lcd.print("settings");
  }
  delay(2000);
  SPIFFS.begin();
  loadConfig();

  if(workMode==0){
    initAP();                         //функция инициализации точки доступа и web-интерфейса
  }
  else{
    //попытка подключения датчика
     while (!tcs.begin()) {
        timeOut++;
        if (timeOut == 30)
        {
          lcd.print("SensorNotFound");
        }
        delay(500);
     } 
    servo2.attach(SERVO2_PIN);
    servo2.write(0);
    delay(2000);
    servo1.attach(SERVO1_PIN);
    servo1.write(0);
    stepper1.setSpeed(10);
    timeOut = 0;
    WiFi.enableInsecureWEP();
    for (int i=0; i<3; i++){
      WiFi.begin(ssid[i], password[i]);   //подключаемся к точке доступа с заданными именем и паролем
      while (WiFi.status() != WL_CONNECTED) //ожидаем подключение, статус WL_CONNECTED означает что соединение установлено
      {
        timeOut++;
        if (timeOut == 30)
          {
            timeOut=0;
            break;
          }
        delay(500);
      }
      if (WiFi.status() == WL_CONNECTED){
        lcd.clear();
        lcd.print("Connected");
        lcd.setCursor(0,1);
        lcd.print(ssid[i]);
        break;
      }
    }
      if (WiFi.status() != WL_CONNECTED) {
        lcd.clear();
        lcd.print("WiFi Not Found");
        delay(10000);
        lcd.noBacklight(); 
        ESP.deepSleep(workDelay*sleepConst);
      }
    
  }

}

void loop() {
  if(workMode==0){
    HTTP.handleClient();    // Обработчик HTTP-событий 

  }
  else {
    int rainValue = analogRead(A0);   //Если <900 - идет дождь
    if(rainValue>=900){
      takeData();                     //функция сбора и отпраки данных
    lcd.noBacklight(); 
    ESP.deepSleep(workDelay*sleepConst);
    }
    
  }
}

void takeData(){
  //int counter = 0;
  int angleCounter = 0;
  servo1.write(180);
  delay(2000);
  servo2.write(180);
  
  //начало формирования запроса
  
  String json;
  json  =  "{\"system\":{ ";
  json += "\"Akey\":\"NeKKxx1\", ";
  json += "\"Serial\":\"01\", ";
  json += "\"Version\":\"2022-05-10\", ";
  json += "\"RSSI\":\"" + String(WiFi.RSSI()) + "\",";
  json += "\"MAC\":\"" + String(WiFi.macAddress()) + "\",";
  IPAddress myIP = WiFi.localIP();
  json += "\"IP\":\""; json+= (String)myIP[0] + String(".") + (String)myIP[1] + String(".") +(String)myIP[2] + String(".") + (String)myIP[3]; json += "\"}";
  
  for (int i=0; i<4; i++){
    //getData();
    uint16_t r, g, b, c, colorTemp, lux;
    tcs.getRawData(&r, &g, &b, &c);
    colorTemp = tcs.calculateColorTemperature_dn40(r, g, b, c);
    lux = tcs.calculateLux(r, g, b);
    //записать в запрос
    json += ",\"AngleData_"+String(angleCounter)+"\":{ ";
    json += "\"CT\":"+String(colorTemp)+",";
    json += "\"lux\":"+String(lux);
    json += "} ";
    stepper1.step(stepCount);
    //counter+=stepCount;
    angleCounter +=stepAngle;
    delay(1000);
  }

  //counter = 0;
  for (int i=0; i<4; i++){
    stepper1.step(-512);
    //counter+=stepCount;
    delay(2000);
  }
  servo2.write(0);
  delay(5000);
  servo1.write(0);
  delay(5000);

  json += "}";
  //отправить запрос
  WiFiClient client;
  if (WiFi.status() == WL_CONNECTED){
    http.begin(client, "http://188.35.161.31/core/jsonadd.php");
    http.addHeader("Content-Type", "application/json");
    int resulthttp = http.POST(json);
    //Serial.print("Done: ");
    //Serial.println(resulthttp);
    http.end();
  }
}


void initAP(){
  WiFi.softAP(AP_ssid/*, AP_password*/);    // Создаём точку доступа
  //Serial.println();
  //Serial.println(WiFi.softAPIP());

  HTTP.begin();
  
  HTTP.on("/setServer",handleSetServer);
  HTTP.on("/setDelay", handleSetDelay);
  HTTP.on("/getConfig", handleGetConfig);
  HTTP.onNotFound(handleNotFound);

  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print(AP_ssid);
  lcd.setCursor(0,1);
  lcd.print("192.168.4.1");
}

void handleNotFound(){
  if(SPIFFS.exists("/index.html")){
      File httpFile = SPIFFS.open("/index.html","r");
      size_t sent = HTTP.streamFile(httpFile, "text/html");                   //  Выводим содержимое файла по HTTP, указывая заголовок типа содержимого contentType
      httpFile.close(); 
    }
    else 
      HTTP.send(404, "text/plain", "Not Found");
}

void handleSetServer(){
  if (HTTP.arg("num")=="1"){
    ssid[0] = HTTP.arg("ssid");
    password[0] = HTTP.arg("pass");
  }
  if (HTTP.arg("num")=="2"){
    ssid[1] = HTTP.arg("ssid");
    password[1] = HTTP.arg("pass");
  }
  if (HTTP.arg("num")=="3"){
    ssid[2] = HTTP.arg("ssid");
    password[2] = HTTP.arg("pass");
  }
  saveConfig();
  //Serial.print("saved");
  HTTP.send(200, "text/plain", "");
}

void handleSetDelay(){
  String buff = HTTP.arg("delay");
  workDelay = convertToInt(buff);
  saveConfig();
  //Serial.print("saved");
  HTTP.send(200, "text/plain", "");
}

//формироание запроса для выдачи на страницу настроек
/*void handleGetConfig(){
  //Serial.println ("Forming JSON");
  String json ="{";
  json += "\"ssid_1\":\"";
  json += ssid[0];
  json += "\",\"ssid_2\":\"";
  json += ssid[1];
  json += "\",\"ssid_3\":\"";
  json += ssid[2];
  json += "\",\"password_1\":\"";
  json += password[0];
  json += "\",\"password_2\":\"";
  json += password[1];
  json += "\",\"password_3\":\"";
  json += password[2];
  json += "\",\"delay\":\"";
  json += String(workDelay);
  json+= "\"}";
  HTTP.send(200, "text/json", json);
}*/


// Загрузка данных сохраненных в файл  config.json
bool loadConfig() {
  // Открываем файл для чтения
  File configFile = SPIFFS.open("/config.json", "r");
  if (!configFile) {
  // если файл не найден  
    //Serial.println("Failed to open config file");
  //  Создаем файл запиав в него данные по умолчанию
    saveConfig();
    return false;
  }
  // Проверяем размер файла, будем использовать файл размером меньше 1024 байта
  size_t size = configFile.size();
  if (size > 1024) {
    //Serial.println("Config file size is too large");
    return false;
  }

// загружаем файл конфигурации в глобальную переменную
  jsonConfig = configFile.readString();
  // Резервируем памяь для json обекта буфер может рости по мере необходимти предпочтительно для ESP8266 
    DynamicJsonBuffer jsonBuffer;
  //  вызовите парсер JSON через экземпляр jsonBuffer
  //  строку возьмем из глобальной переменной String jsonConfig
    JsonObject& root = jsonBuffer.parseObject(jsonConfig);
    configFile.close();
  // Теперь можно получить значения из root  
    ssid[0] = root["ssid_1"].as<String>(); // Так получаем строку
    password[0] = root["password_1"].as<String>();
    ssid[1] = root["ssid_2"].as<String>(); // Так получаем строку
    password[1] = root["password_2"].as<String>();
    ssid[2] = root["ssid_3"].as<String>(); // Так получаем строку
    password[2] = root["password_3"].as<String>();
    workDelay = root["delay"];
    return true;
}

// Запись данных в файл config.json
bool saveConfig() {
  // Резервируем памяь для json обекта буфер может рости по мере необходимти предпочтительно для ESP8266 
  DynamicJsonBuffer jsonBuffer;
  //  вызовите парсер JSON через экземпляр jsonBuffer
  JsonObject& json = jsonBuffer.parseObject(jsonConfig);
  // Заполняем поля json 
  json["ssid_1"] = ssid[0];
  json["password_1"] = password[0];
  json["ssid_2"] = ssid[1];
  json["password_2"] = password[1];
  json["ssid_3"] = ssid[2];
  json["password_3"] = password[2];
  json["delay"] = workDelay;
  // Помещаем созданный json в глобальную переменную json.printTo(jsonConfig);
  json.printTo(jsonConfig);
  // Открываем файл для записи
  File configFile = SPIFFS.open("/config.json", "w");
  if (!configFile) {
    //Serial.println("Failed to open config file for writing");
    return false;
  }
  // Записываем строку json в файл 
  json.printTo(configFile);
  configFile.close();
  return true;
  }

  int convertToInt(String num){
    int len = num.length();
    int result;
    for(int i=0; i<len; i++)
    {
      result = result * 10 + ( num[i] - '0' );
    }
    return result;
  }
